"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ErrorHanding = require("../util/error-handling");
const base_node_1 = require("./base-node");
const InputNodeScanner_1 = require("./InputNodeScanner");
var RED;
function setRED(globalRED) {
    RED = globalRED;
}
exports.setRED = setRED;
class InputNodeImpl extends base_node_1.NodeBaseImpl {
    constructor(nodeConfig, deviceConfig, node) {
        super(nodeConfig, deviceConfig, node);
        this.onClose = () => {
            this.inputNodeHelper.close();
            if (this.requestDelayedTimer) {
                clearTimeout(this.requestDelayedTimer);
            }
        };
        this.scanNotDoneCallback = () => {
            if (!this.previousResponseError) {
                this.node.status({ fill: "yellow", shape: "dot", text: "scanning (delayed)" });
            }
            this.requestDelayedTimer = null;
        };
        this.onScan = () => {
            this.requestDelayedTimer = setTimeout(this.scanNotDoneCallback, 3000);
            var reqInfo = this.getReadRequest();
            if (!reqInfo || !reqInfo.promise) {
                this.node.status({ fill: "red", shape: "dot", text: "error" });
                return;
            }
            reqInfo.promise.then((fullfilledResponse) => {
                clearTimeout(this.requestDelayedTimer);
                this.requestDelayedTimer = null;
                this.node.status({ fill: "green", shape: "dot", text: "scanning" });
                this.previousResponseError = undefined;
                let newValue = fullfilledResponse.body[reqInfo.valueName];
                if (this.nodeInputConfig.dataType == 'mmp-address') {
                    if (this.nodeInputConfig.mmpType == 'string' || reqInfo.mmpLength === 1) {
                        newValue = newValue[0];
                    }
                }
                if (this.inputNodeHelper.updateValue(newValue)) {
                    let msg = {
                        payload: newValue,
                        body: fullfilledResponse.body,
                        inputType: this.nodeInputConfig.dataType
                    };
                    this.node.send(msg);
                }
            }, (error) => {
                clearTimeout(this.requestDelayedTimer);
                this.requestDelayedTimer = null;
                this.previousResponseError = ErrorHanding.handleErrorResponse(error, {}, this.node, this.previousResponseError);
                this.inputNodeHelper.updateError();
            });
        };
        this.nodeInputConfig = nodeConfig;
        var deadband;
        var nodeChangeType = InputNodeScanner_1.InputNodeChangeType.None;
        this.scanTimeMs = parseFloat(nodeConfig.scanTimeSec);
        if (isNaN(this.scanTimeMs))
            this.scanTimeMs = 500;
        else
            this.scanTimeMs = this.scanTimeMs * 1000.0;
        if ((this.nodeInputConfig.dataType == 'channel-analog') ||
            (this.nodeInputConfig.dataType == 'channel-digital-feature-value') ||
            (this.nodeInputConfig.dataType == 'mmp-address' && this.nodeInputConfig.mmpType != 'string')) {
            nodeChangeType = InputNodeScanner_1.InputNodeChangeType.Deadband;
            deadband = parseFloat(nodeConfig.deadband);
            if (isNaN(deadband))
                deadband = 1;
        }
        if ((this.nodeInputConfig.dataType == 'channel-digital-on-latch') ||
            (this.nodeInputConfig.dataType == 'channel-digital-off-latch') ||
            (this.nodeInputConfig.dataType == 'channel-digital-turn-on')) {
            nodeChangeType = InputNodeScanner_1.InputNodeChangeType.RisingEdgeOnly;
        }
        if (this.nodeInputConfig.dataType == 'channel-digital-turn-off') {
            nodeChangeType = InputNodeScanner_1.InputNodeChangeType.FallingEdgeOnly;
        }
        this.inputNodeHelper = new InputNodeScanner_1.InputNodeScanner(this.scanTimeMs, nodeChangeType, deadband, nodeConfig.sendInitialValue, this.onScan);
        if (this.apiClient) {
            if (this.scanTimeMs > 0) {
                this.inputNodeHelper.startScan();
            }
        }
        else {
            this.node.status({ fill: "red", shape: "dot", text: "No device" });
        }
    }
    static getNodeType() {
        return 'groov-io-input';
    }
    getReadRequest() {
        var nodeConfig = this.nodeInputConfig;
        switch (nodeConfig.dataType) {
            case 'channel-digital-state':
            case 'channel-digital-turn-on':
            case 'channel-digital-turn-off':
                return {
                    promise: this.apiClient.getChannelDigitalStatus('local', this.moduleIndex, this.channelIndex),
                    dataType: nodeConfig.dataType,
                    valueName: 'state'
                };
            case 'channel-digital-on-latch':
                return {
                    promise: this.apiClient.getChannelDigitalStatus('local', this.moduleIndex, this.channelIndex),
                    dataType: nodeConfig.dataType,
                    valueName: 'onLatchState'
                };
            case 'channel-digital-off-latch':
                return {
                    promise: this.apiClient.getChannelDigitalStatus('local', this.moduleIndex, this.channelIndex),
                    dataType: nodeConfig.dataType,
                    valueName: 'offLatchState'
                };
            case 'channel-digital-feature-value':
                return {
                    promise: this.apiClient.getChannelDigitalStatus('local', this.moduleIndex, this.channelIndex),
                    dataType: nodeConfig.dataType,
                    valueName: 'featureValue'
                };
            case 'channel-analog':
                return {
                    promise: this.apiClient.getChannelAnalogStatus('local', this.moduleIndex, this.channelIndex),
                    dataType: nodeConfig.dataType,
                    valueName: 'value'
                };
            case 'mmp-address':
                return {
                    promise: this.apiClient.getMmpValues('local', this.nodeConfig.mmpAddress, this.nodeConfig.mmpType, this.mmpLength, this.nodeConfig.mmpEncoding),
                    dataType: nodeConfig.dataType,
                    valueName: 'mmpValues',
                    mmpLength: this.mmpLength
                };
        }
    }
}
exports.InputNodeImpl = InputNodeImpl;
function createInputNode(nodeConfig, returnImpl) {
    var node = this;
    RED.nodes.createNode(node, nodeConfig);
    var deviceConfig = RED.nodes.getNode(nodeConfig.device);
    var impl = new InputNodeImpl(nodeConfig, deviceConfig, node);
    node.on('close', impl.onClose);
    if (returnImpl) {
        return impl;
    }
}
exports.createInputNode = createInputNode;
