"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ErrorHanding = require("../util/error-handling");
const base_node_1 = require("./base-node");
var RED;
function setRED(globalRED) {
    RED = globalRED;
}
exports.setRED = setRED;
class ReadNodeImpl extends base_node_1.FunctionNodeBaseImpl {
    constructor(nodeConfig, deviceConfig, node) {
        super(nodeConfig, deviceConfig, node);
        this.scanNotDoneCallback = () => {
            if (!this.previousResponseError) {
                this.node.status({ fill: "yellow", shape: "dot", text: "reading (delayed)" });
            }
            this.requestDelayedTimer = null;
        };
        this.onInput = (msg) => {
            this.requestDelayedTimer = setTimeout(this.scanNotDoneCallback, 3000);
            if (this.previousResponseError) {
                this.node.status({
                    fill: "red", shape: "dot", text: "reading [" +
                        this.previousResponseError.nodeShortErrorMsg + "]"
                });
            }
            else {
                this.node.status({ fill: "green", shape: "dot", text: "reading" });
            }
            var reqInfo;
            if (this.apiClient) {
                reqInfo = this.getReadRequest(msg);
            }
            else {
                this.node.status({ fill: "red", shape: "dot", text: "No device" });
                return;
            }
            if (!reqInfo || !reqInfo.promise) {
                this.node.status({ fill: "red", shape: "dot", text: "error" });
                return;
            }
            reqInfo.promise.then((fullfilledResponse) => {
                this.clearRequestDelayedTimer();
                this.previousResponseError = undefined;
                this.node.status({});
                msg.body = fullfilledResponse.body;
                this.setValue(msg, fullfilledResponse, reqInfo);
                this.node.send(msg);
                var queueLength = this.msgQueue.done(0);
                this.updateQueuedStatus(queueLength);
            }, (error) => {
                this.clearRequestDelayedTimer();
                this.previousResponseError = ErrorHanding.handleErrorResponse(error, msg, this.node, this.previousResponseError);
                this.msgQueue.done(50);
            });
        };
        this.nodeReadConfig = nodeConfig;
        this.node.on('close', () => {
            this.onClose();
        });
        this.node.on('input', (msg) => {
            this.addMsg(msg);
        });
    }
    static getNodeType() {
        return 'groov-io-read';
    }
    onClose() {
        this.clearRequestDelayedTimer();
    }
    clearRequestDelayedTimer() {
        if (this.requestDelayedTimer) {
            clearTimeout(this.requestDelayedTimer);
            this.requestDelayedTimer = null;
        }
    }
    setValue(msg, fullfilledResponse, reqInfo) {
        var newValue;
        if (reqInfo.valueName) {
            newValue = fullfilledResponse.body[reqInfo.valueName];
        }
        else {
            newValue = fullfilledResponse.body;
        }
        if (reqInfo.dataType == 'mmp-address') {
            if (this.nodeReadConfig.mmpType == 'string' || reqInfo.mmpLength === 1) {
                newValue = newValue[0];
            }
        }
        switch (this.nodeReadConfig.valueType) {
            case 'msg':
                RED.util.setMessageProperty(msg, this.nodeReadConfig.value, newValue, true);
                ;
                break;
            case 'msg.payload':
                msg.payload = newValue;
                break;
            default:
                throw new Error('Unexpected value type - ' + this.nodeReadConfig.valueType);
        }
    }
    getReadRequest(msg) {
        var nodeConfig = this.nodeConfig;
        var moduleIndex = msg.moduleIndex === undefined ? this.moduleIndex : msg.moduleIndex;
        var channelIndex = msg.channelIndex === undefined ? this.channelIndex : msg.channelIndex;
        var mmpAddress = msg.mmpAddress === undefined ? nodeConfig.mmpAddress : msg.mmpAddress;
        switch (nodeConfig.dataType) {
            case 'channel-digital':
                return {
                    promise: this.apiClient.getChannelDigitalStatus('local', moduleIndex, channelIndex),
                    dataType: nodeConfig.dataType,
                    valueName: 'state'
                };
            case 'channel-analog':
                return {
                    promise: this.apiClient.getChannelAnalogStatus('local', moduleIndex, channelIndex),
                    dataType: nodeConfig.dataType,
                    valueName: 'value'
                };
            case 'channel-config':
                return {
                    promise: this.apiClient.getChannelConfiguration('local', moduleIndex, channelIndex),
                    dataType: nodeConfig.dataType,
                };
            case 'module-digital':
                return {
                    promise: this.apiClient.getModuleDigitalChannelValues('local', moduleIndex),
                    dataType: nodeConfig.dataType,
                    valueName: 'channelValues'
                };
            case 'module-analog':
                return {
                    promise: this.apiClient.getModuleAnalogChannelValues('local', moduleIndex),
                    dataType: nodeConfig.dataType,
                    valueName: 'channelValues'
                };
            case 'modules-quality':
                return {
                    promise: this.apiClient.getModuleQualityFlags('local'),
                    dataType: nodeConfig.dataType,
                    valueName: 'qualityMask'
                };
            case 'mmp-address':
                return {
                    promise: this.apiClient.getMmpValues('local', mmpAddress, this.nodeConfig.mmpType, this.mmpLength, this.nodeConfig.mmpEncoding),
                    dataType: nodeConfig.dataType,
                    valueName: 'mmpValues',
                    mmpLength: this.mmpLength
                };
        }
    }
}
exports.ReadNodeImpl = ReadNodeImpl;
function createReadNode(nodeConfig) {
    RED.nodes.createNode(this, nodeConfig);
    var deviceConfig = RED.nodes.getNode(nodeConfig.device);
    var node = this;
    var impl = new ReadNodeImpl(nodeConfig, deviceConfig, node);
}
exports.createReadNode = createReadNode;
