"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ErrorHanding = require("../util/error-handling");
const base_node_1 = require("./base-node");
var RED;
function setRED(globalRED) {
    RED = globalRED;
}
exports.setRED = setRED;
class WriteNodeImpl extends base_node_1.FunctionNodeBaseImpl {
    constructor(nodeConfig, deviceConfig, node) {
        super(nodeConfig, deviceConfig, node);
        this.scanNotDoneCallback = () => {
            if (!this.previousResponseError) {
                this.node.status({ fill: "yellow", shape: "dot", text: "writing (delayed)" });
            }
            this.requestDelayedTimer = null;
        };
        this.onInput = (msg) => {
            try {
                var valueFinal = WriteNodeImpl.getValueToWrite(msg, this.nodeWriteConfig);
            }
            catch (e) {
                var errorMessage;
                if (e instanceof Error)
                    errorMessage = e.message;
                else
                    errorMessage = JSON.stringify(e);
                this.node.error(errorMessage, msg);
                this.node.status({ fill: "red", shape: "dot", text: "error" });
                this.msgQueue.done(0);
                return;
            }
            this.requestDelayedTimer = setTimeout(this.scanNotDoneCallback, 3000);
            if (this.previousResponseError) {
                this.node.status({
                    fill: "red", shape: "dot", text: "writing [" +
                        this.previousResponseError.nodeShortErrorMsg + "]"
                });
            }
            else {
                this.node.status({ fill: "green", shape: "dot", text: "writing" });
            }
            var promise;
            if (this.apiClient) {
                promise = this.getWriteRequest(msg, valueFinal);
            }
            else {
                this.node.status({ fill: "red", shape: "dot", text: "No device" });
                return;
            }
            if (!promise) {
                this.node.status({ fill: "red", shape: "dot", text: "error" });
                return;
            }
            promise.then((fullfilledResponse) => {
                this.clearRequestDelayedTimer();
                this.previousResponseError = undefined;
                WriteNodeImpl.activeMessageCount--;
                this.node.status({});
                msg.body = fullfilledResponse.body;
                this.node.send(msg);
                var queueLength = this.msgQueue.done(0);
                this.updateQueuedStatus(queueLength);
            }, (error) => {
                this.clearRequestDelayedTimer();
                WriteNodeImpl.activeMessageCount--;
                this.previousResponseError = ErrorHanding.handleErrorResponse(error, msg, this.node, this.previousResponseError);
                this.msgQueue.done(50);
            });
        };
        this.nodeWriteConfig = nodeConfig;
        this.node.on('close', () => {
            this.onClose();
        });
        this.node.on('input', (msg) => {
            this.addMsg(msg);
        });
    }
    static getNodeType() {
        return 'groov-io-write';
    }
    onClose() {
        this.clearRequestDelayedTimer();
    }
    clearRequestDelayedTimer() {
        if (this.requestDelayedTimer) {
            clearTimeout(this.requestDelayedTimer);
            this.requestDelayedTimer = null;
        }
    }
    static getValueToWrite(msg, nodeConfig) {
        var valueFinal = null;
        switch (nodeConfig.valueType) {
            case 'msg':
            case 'msg.payload':
                var msgProperty;
                if (nodeConfig.valueType === 'msg.payload') {
                    msgProperty = 'payload';
                }
                else {
                    msgProperty = nodeConfig.value;
                }
                var msgValue = RED.util.getMessageProperty(msg, msgProperty);
                if (msgValue === undefined) {
                    throw new Error('msg.' + msgProperty + ' is undefined.');
                }
                if (typeof msgValue === 'string') {
                    valueFinal = WriteNodeImpl.stringValueToWriteValue(nodeConfig.dataType, msgValue);
                }
                else {
                    valueFinal = msgValue;
                }
                break;
            case 'value':
                var mmpType;
                if (nodeConfig.dataType === 'mmp-address')
                    mmpType = nodeConfig.mmpType;
                valueFinal = WriteNodeImpl.stringValueToWriteValue(nodeConfig.dataType, nodeConfig.value, mmpType);
                break;
            default:
                throw new Error('Unexpected value type - ' + nodeConfig.valueType);
        }
        return valueFinal;
    }
    static stringValueToWriteValue(dataType, value, mmpType) {
        if (typeof value !== 'string')
            throw new Error('Invalid Input');
        var writeVal = null;
        var isString = false;
        var isInteger = false;
        var isFloat = false;
        var isDigital = (dataType === 'channel-digital');
        if (dataType === 'channel-digital')
            isDigital = true;
        else if (dataType === 'channel-analog')
            isFloat = true;
        else if (dataType === 'mmp-address') {
            if (mmpType) {
                switch (mmpType) {
                    case 'int8':
                    case 'uint8':
                    case 'int32':
                    case 'uint32':
                        isInteger = true;
                        break;
                    case 'float':
                        isFloat = true;
                        break;
                    case 'string':
                        isString = true;
                }
            }
        }
        if (isDigital) {
            var result = false;
            var testValue = value.toLowerCase().trim();
            if ((testValue === 'off') || (testValue === 'false'))
                result = false;
            else if ((testValue === 'on') || (testValue === 'true'))
                result = true;
            else
                throw new Error('"' + value + '" is not a valid value for a digital output.');
            writeVal = result;
        }
        else if (isFloat || isInteger) {
            var valueTrimmed = value.trim();
            if (valueTrimmed === '') {
                throw new Error('"' + value + '" is not a valid number.');
            }
            var valueAsNumber;
            if (isFloat)
                valueAsNumber = Number.parseFloat(valueTrimmed);
            else if (isInteger)
                valueAsNumber = Number.parseInt(valueTrimmed);
            if (isNaN(valueAsNumber)) {
                throw new Error('"' + value + '" is not a valid number.');
            }
            else {
                writeVal = valueAsNumber;
            }
        }
        else if (isString) {
            writeVal = value;
        }
        else {
            writeVal = value;
        }
        return writeVal;
    }
    getWriteRequest(msg, value) {
        var nodeConfig = this.nodeConfig;
        var moduleIndex = msg.moduleIndex === undefined ? this.moduleIndex : msg.moduleIndex;
        var channelIndex = msg.channelIndex === undefined ? this.channelIndex : msg.channelIndex;
        var mmpAddress = msg.mmpAddress === undefined ? nodeConfig.mmpAddress : msg.mmpAddress;
        switch (nodeConfig.dataType) {
            case 'channel-digital':
                return this.apiClient.setDigitalChannelState('local', moduleIndex, channelIndex, { value: value });
            case 'channel-analog':
                return this.apiClient.setAnalogChannelValue('local', moduleIndex, channelIndex, { value: value });
            case 'channel-clear-on-latch':
                return this.apiClient.clearDigitalChannelOnLatch('local', moduleIndex, channelIndex);
            case 'channel-clear-off-latch':
                return this.apiClient.clearDigitalChannelOffLatch('local', moduleIndex, channelIndex);
            case 'channel-clear-feature-value':
                return this.apiClient.clearDigitalChannelFeature('local', moduleIndex, channelIndex);
            case 'channel-clear-min-value':
                return this.apiClient.clearAnalogChannelMinValue('local', moduleIndex, channelIndex);
            case 'channel-clear-max-value':
                return this.apiClient.clearAnalogChannelMaxValue('local', moduleIndex, channelIndex);
            case 'channel-counter-start':
                return this.apiClient.setDigitalChannelCounterActive('local', moduleIndex, channelIndex, { value: true });
            case 'channel-counter-stop':
                return this.apiClient.setDigitalChannelCounterActive('local', moduleIndex, channelIndex, { value: false });
            case 'channel-config':
                return this.apiClient.setChannelConfiguration('local', moduleIndex, channelIndex, value);
            case 'mmp-address':
                let body = {
                    value: value,
                    type: this.nodeConfig.mmpType
                };
                if (this.nodeConfig.mmpType == 'string') {
                    body['encoding'] = this.nodeConfig.mmpEncoding;
                }
                return this.apiClient.setMmpValue('local', mmpAddress, body);
        }
    }
}
WriteNodeImpl.activeMessageCount = 0;
exports.WriteNodeImpl = WriteNodeImpl;
function createWriteNode(nodeConfig) {
    RED.nodes.createNode(this, nodeConfig);
    var deviceConfig = RED.nodes.getNode(nodeConfig.device);
    var node = this;
    var impl = new WriteNodeImpl(nodeConfig, deviceConfig, node);
}
exports.createWriteNode = createWriteNode;
